@@ -5,13 +5,12 @@
 * Perform PPPoE discovery
 *
 * Copyright (C) 1999-2015 by Roaring Penguin Software Inc.
+* Copyright (C) 2018-2020 Dianne Skoll
 *
 * LIC: GPL
 *
 ***********************************************************************/
 
-static char const RCSID[] =
-"$Id$";
 #define _GNU_SOURCE 1
 
 #include "pppoe.h"
@@ -58,6 +57,34 @@ struct HostUniqInfo {
     int forMe;
 };
 
+/* Calculate time remaining until *expire_at into *tv, returns 0 if now >= *expire_at */
+static int
+time_left(struct timeval *tv, struct timeval *expire_at)
+{
+    struct timeval now;
+
+    if (gettimeofday(&now, NULL) < 0) {
+	fatalSys("gettimeofday (time_left)");
+    }
+    tv->tv_sec = expire_at->tv_sec - now.tv_sec;
+    tv->tv_usec = expire_at->tv_usec - now.tv_usec;
+    if (tv->tv_usec < 0) {
+	tv->tv_usec += 1000000;
+	if (tv->tv_sec) {
+	    tv->tv_sec--;
+	} else {
+	    /* Timed out */
+	    return 0;
+	}
+    }
+    if (tv->tv_sec <= 0 && tv->tv_usec <= 0) {
+	/* Timed out */
+	return 0;
+    }
+
+    return 1;
+}
+
 /**********************************************************************
 *%FUNCTION: parseForHostUniq
 *%ARGUMENTS:
@@ -225,8 +252,8 @@ parsePADOTags(UINT16_t type, UINT16_t len, unsigned char *data,
 	    mru = ntohs(mru);
 	    if (mru >= ETH_PPPOE_MTU) {
 		if (lcp_allowoptions[0].mru > mru) lcp_allowoptions[0].mru = mru;
-               if (lcp_wantoptions[0].mru > mru) lcp_wantoptions[0].mru = mru;
-               conn->seenMaxPayload = 1;
+		if (lcp_wantoptions[0].mru > mru) lcp_wantoptions[0].mru = mru;
+		conn->seenMaxPayload = 1;
 	    }
 	}
 	break;
@@ -276,8 +303,8 @@ parsePADSTags(UINT16_t type, UINT16_t len, unsigned char *data,
 	    mru = ntohs(mru);
 	    if (mru >= ETH_PPPOE_MTU) {
 		if (lcp_allowoptions[0].mru > mru) lcp_allowoptions[0].mru = mru;
-               if (lcp_wantoptions[0].mru > mru) lcp_wantoptions[0].mru = mru;
-               conn->seenMaxPayload = 1;
+		if (lcp_wantoptions[0].mru > mru) lcp_wantoptions[0].mru = mru;
+		conn->seenMaxPayload = 1;
 	    }
 	}
 	break;
@@ -316,8 +343,7 @@ sendPADI(PPPoEConnection *conn)
     memcpy(packet.ethHdr.h_source, conn->myEth, ETH_ALEN);
 
     packet.ethHdr.h_proto = htons(Eth_PPPOE_Discovery);
-    packet.ver = 1;
-    packet.type = 1;
+    packet.vertype = PPPOE_VER_TYPE(1, 1);
     packet.code = CODE_PADI;
     packet.session = 0;
 
@@ -393,7 +419,6 @@ waitForPADO(PPPoEConnection *conn, int timeout)
     int r;
     struct timeval tv;
     struct timeval expire_at;
-    struct timeval now;
 
     PPPoEPacket packet;
     int len;
@@ -411,21 +436,7 @@ waitForPADO(PPPoEConnection *conn, int timeout)
 
     do {
 	if (BPF_BUFFER_IS_EMPTY) {
-	    if (gettimeofday(&now, NULL) < 0) {
-		fatalSys("gettimeofday (waitForPADO)");
-	    }
-	    tv.tv_sec = expire_at.tv_sec - now.tv_sec;
-	    tv.tv_usec = expire_at.tv_usec - now.tv_usec;
-	    if (tv.tv_usec < 0) {
-		tv.tv_usec += 1000000;
-		if (tv.tv_sec) {
-		    tv.tv_sec--;
-		} else {
-		    /* Timed out */
-		    return;
-		}
-	    }
-	    if (tv.tv_sec <= 0 && tv.tv_usec <= 0) {
+	    if(!time_left(&tv, &expire_at)) {
 		/* Timed out */
 		return;
 	    }
@@ -472,8 +483,8 @@ waitForPADO(PPPoEConnection *conn, int timeout)
 	if (!packetIsForMe(conn, &packet)) continue;
 
 	if (packet.code == CODE_PADO) {
-	    if (NOT_UNICAST(packet.ethHdr.h_source)) {
-		printErr("Ignoring PADO packet from non-unicast MAC address");
+	    if (BROADCAST(packet.ethHdr.h_source)) {
+		printErr("Ignoring PADO packet from broadcast MAC address");
 		continue;
 	    }
 #ifdef PLUGIN
@@ -552,8 +563,7 @@ sendPADR(PPPoEConnection *conn)
     memcpy(packet.ethHdr.h_source, conn->myEth, ETH_ALEN);
 
     packet.ethHdr.h_proto = htons(Eth_PPPOE_Discovery);
-    packet.ver = 1;
-    packet.type = 1;
+    packet.vertype = PPPOE_VER_TYPE(1, 1);
     packet.code = CODE_PADR;
     packet.session = 0;
 
@@ -637,7 +647,6 @@ waitForPADS(PPPoEConnection *conn, int timeout)
     int r;
     struct timeval tv;
     struct timeval expire_at;
-    struct timeval now;
 
     PPPoEPacket packet;
     int len;
@@ -649,21 +658,7 @@ waitForPADS(PPPoEConnection *conn, int timeout)
 
     do {
 	if (BPF_BUFFER_IS_EMPTY) {
-	    if (gettimeofday(&now, NULL) < 0) {
-		fatalSys("gettimeofday (waitForPADS)");
-	    }
-	    tv.tv_sec = expire_at.tv_sec - now.tv_sec;
-	    tv.tv_usec = expire_at.tv_usec - now.tv_usec;
-	    if (tv.tv_usec < 0) {
-		tv.tv_usec += 1000000;
-		if (tv.tv_sec) {
-		    tv.tv_sec--;
-		} else {
-		    /* Timed out */
-		    return;
-		}
-	    }
-	    if (tv.tv_sec <= 0 && tv.tv_usec <= 0) {
+	    if(!time_left(&tv, &expire_at)) {
 		/* Timed out */
 		return;
 	    }
@@ -766,12 +761,12 @@ discovery(PPPoEConnection *conn)
     do {
 	padiAttempts++;
 	if (padiAttempts > MAX_PADI_ATTEMPTS) {
+	    printErr("Timeout waiting for PADO packets");
 	    if (persist) {
 		padiAttempts = 0;
 		timeout = conn->discoveryTimeout;
-		printErr("Timeout waiting for PADO packets");
 	    } else {
-		rp_fatal("Timeout waiting for PADO packets");
+		return;
 	    }
 	}
 	sendPADI(conn);
@@ -799,14 +794,14 @@ discovery(PPPoEConnection *conn)
     do {
 	padrAttempts++;
 	if (padrAttempts > MAX_PADI_ATTEMPTS) {
+	    printErr("Timeout waiting for PADS packets");
 	    if (persist) {
 		padrAttempts = 0;
 		timeout = conn->discoveryTimeout;
-		printErr("Timeout waiting for PADS packets");
 		/* Go back to sending PADI again */
 		goto SEND_PADI;
 	    } else {
-		rp_fatal("Timeout waiting for PADS packets");
+		return;
 	    }
 	}
 	sendPADR(conn);
