@@ -4,7 +4,8 @@
 *
 * Implementation of a user-space PPPoE server
 *
-* Copyright (C) 2000 Roaring Penguin Software Inc.
+* Copyright (C) 2000-2012 Roaring Penguin Software Inc.
+* Copyright (C) 2018-2020 Dianne Skoll
 *
 * This program may be distributed according to the terms of the GNU
 * General Public License, version 2 or (at your option) any later version.
@@ -15,17 +16,19 @@
 *
 ***********************************************************************/
 
-static char const RCSID[] =
-"$Id$";
-
 #include "config.h"
 
+#include <sys/socket.h>
+#if defined(HAVE_LINUX_IF_H)
+#include <linux/if.h>
+#elif defined(HAVE_NET_IF_H)
+#include <net/if.h>
+#endif
+
 #if defined(HAVE_NETPACKET_PACKET_H) || defined(HAVE_LINUX_IF_PACKET_H)
 #define _POSIX_SOURCE 1 /* For sigaction defines */
 #endif
 
-#define _BSD_SOURCE 1 /* for gethostname */
-
 #include "pppoe-server.h"
 #include "md5.h"
 
@@ -37,6 +40,7 @@ static char const RCSID[] =
 #include <string.h>
 #include <stdlib.h>
 #include <fcntl.h>
+#include <sys/file.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -132,15 +136,30 @@ EventSelector *event_selector;
 /* Use Linux kernel-mode PPPoE? */
 static int UseLinuxKernelModePPPoE = 0;
 
+/* Requested max_ppp_payload */
+static UINT16_t max_ppp_payload = 0;
+
 /* File with PPPD options */
 static char *pppoptfile = NULL;
 
+static char *pppd_path = PPPD_PATH;
+static char *pppoe_path = PPPOE_PATH;
+
+static char *motd_string = NULL;
+static char *hurl_string = NULL;
+
 static int Debug = 0;
 static int CheckPoolSyntax = 0;
 
 /* Synchronous mode */
 static int Synchronous = 0;
 
+/* Ignore PADI if no free sessions */
+static int IgnorePADIIfNoFreeSessions = 0;
+
+static int KidPipe[2] = {-1, -1};
+static int LockFD = -1;
+
 /* Random seed for cookie generation */
 #define SEED_LEN 16
 #define MD5_LEN 16
@@ -227,7 +246,7 @@ childHandler(pid_t pid, int status, void *s)
 #endif
 
     memset(&conn, 0, sizeof(conn));
-    conn.useHostUniq = 0;
+    conn.hostUniq = NULL;
 
     syslog(LOG_INFO,
 	   "Session %u closed for client "
@@ -420,6 +439,15 @@ parsePADITags(UINT16_t type, UINT16_t len, unsigned char *data,
 	      void *extra)
 {
     switch(type) {
+    case TAG_PPP_MAX_PAYLOAD:
+	if (len == sizeof(max_ppp_payload)) {
+	    memcpy(&max_ppp_payload, data, sizeof(max_ppp_payload));
+	    max_ppp_payload = ntohs(max_ppp_payload);
+	    if (max_ppp_payload <= ETH_PPPOE_MTU) {
+		max_ppp_payload = 0;
+	    }
+	}
+	break;
     case TAG_SERVICE_NAME:
 	/* Copy requested service name */
 	requestedService.type = htons(type);
@@ -456,6 +484,15 @@ parsePADRTags(UINT16_t type, UINT16_t len, unsigned char *data,
 	      void *extra)
 {
     switch(type) {
+    case TAG_PPP_MAX_PAYLOAD:
+	if (len == sizeof(max_ppp_payload)) {
+	    memcpy(&max_ppp_payload, data, sizeof(max_ppp_payload));
+	    max_ppp_payload = ntohs(max_ppp_payload);
+	    if (max_ppp_payload <= ETH_PPPOE_MTU) {
+		max_ppp_payload = 0;
+	    }
+	}
+	break;
     case TAG_RELAY_SESSION_ID:
 	relayId.type = htons(type);
 	relayId.length = htons(len);
@@ -596,6 +633,12 @@ processPADI(Interface *ethif, PPPoEPacket *packet, int len)
 	return;
     }
 
+    /* If no free sessions and "-i" flag given, ignore */
+    if (IgnorePADIIfNoFreeSessions && !FreeSessions) {
+	syslog(LOG_INFO, "PADI ignored - No free session slots available");
+	return;
+    }
+
     /* If number of sessions per MAC is limited, check here and don't
        send PADO if already max number of sessions. */
     if (MaxSessionsPerMac) {
@@ -620,6 +663,8 @@ processPADI(Interface *ethif, PPPoEPacket *packet, int len)
     relayId.type = 0;
     hostUniq.type = 0;
     requestedService.type = 0;
+    max_ppp_payload = 0;
+
     parsePacket(packet, parsePADITags, NULL);
 
     /* If PADI specified non-default service name, and we do not offer
@@ -655,8 +700,7 @@ processPADI(Interface *ethif, PPPoEPacket *packet, int len)
     memcpy(pado.ethHdr.h_dest, packet->ethHdr.h_source, ETH_ALEN);
     memcpy(pado.ethHdr.h_source, myAddr, ETH_ALEN);
     pado.ethHdr.h_proto = htons(Eth_PPPOE_Discovery);
-    pado.ver = 1;
-    pado.type = 1;
+    pado.vertype = PPPOE_VER_TYPE(1, 1);
     pado.code = CODE_PADO;
     pado.session = 0;
     plen = TAG_HDR_SIZE + acname_len;
@@ -665,6 +709,27 @@ processPADI(Interface *ethif, PPPoEPacket *packet, int len)
     memcpy(cursor, &acname, acname_len + TAG_HDR_SIZE);
     cursor += acname_len + TAG_HDR_SIZE;
 
+    /* If we asked for an MTU, handle it */
+    if (max_ppp_payload > ETH_PPPOE_MTU && ethif->mtu > 0) {
+	/* Shrink payload to fit */
+	if (max_ppp_payload > ethif->mtu - TOTAL_OVERHEAD) {
+	    max_ppp_payload = ethif->mtu - TOTAL_OVERHEAD;
+	}
+	if (max_ppp_payload > ETH_JUMBO_LEN - TOTAL_OVERHEAD) {
+	    max_ppp_payload = ETH_JUMBO_LEN - TOTAL_OVERHEAD;
+	}
+	if (max_ppp_payload > ETH_PPPOE_MTU) {
+	    PPPoETag maxPayload;
+	    UINT16_t mru = htons(max_ppp_payload);
+	    maxPayload.type = htons(TAG_PPP_MAX_PAYLOAD);
+	    maxPayload.length = htons(sizeof(mru));
+	    memcpy(maxPayload.payload, &mru, sizeof(mru));
+	    CHECK_ROOM(cursor, pado.payload, sizeof(mru) + TAG_HDR_SIZE);
+	    memcpy(cursor, &maxPayload, sizeof(mru) + TAG_HDR_SIZE);
+	    cursor += sizeof(mru) + TAG_HDR_SIZE;
+	    plen += sizeof(mru) + TAG_HDR_SIZE;
+	}
+    }
     /* If no service-names specified on command-line, just send default
        zero-length name.  Otherwise, add all service-name tags */
     servname.type = htons(TAG_SERVICE_NAME);
@@ -790,6 +855,10 @@ processPADR(Interface *ethif, PPPoEPacket *packet, int len)
     int slen = 0;
     char const *serviceName = NULL;
 
+    /* Temporary structure for sending PADM's. */
+    PPPoEConnection conn;
+
+
 #ifdef HAVE_LICENSE
     int freemem;
 #endif
@@ -824,6 +893,8 @@ processPADR(Interface *ethif, PPPoEPacket *packet, int len)
 	    return;
 	}
     }
+
+    max_ppp_payload = 0;
     parsePacket(packet, parsePADRTags, NULL);
 
     /* Check that everything's cool */
@@ -941,10 +1012,15 @@ processPADR(Interface *ethif, PPPoEPacket *packet, int len)
 	return;
     }
 
-    /* In the child process.  */
+    /* In the child process */
+
+    /* Reset signal handlers to default */
+    signal(SIGTERM, SIG_DFL);
+    signal(SIGINT, SIG_DFL);
 
     /* Close all file descriptors except for socket */
     closelog();
+    if (LockFD >= 0) close(LockFD);
     for (i=0; i<CLOSEFD; i++) {
 	if (i != sock) {
 	    close(i);
@@ -960,8 +1036,7 @@ processPADR(Interface *ethif, PPPoEPacket *packet, int len)
     memcpy(pads.ethHdr.h_dest, packet->ethHdr.h_source, ETH_ALEN);
     memcpy(pads.ethHdr.h_source, myAddr, ETH_ALEN);
     pads.ethHdr.h_proto = htons(Eth_PPPOE_Discovery);
-    pads.ver = 1;
-    pads.type = 1;
+    pads.vertype = PPPOE_VER_TYPE(1, 1);
     pads.code = CODE_PADS;
 
     pads.session = cliSession->sess;
@@ -979,6 +1054,29 @@ processPADR(Interface *ethif, PPPoEPacket *packet, int len)
     cursor += TAG_HDR_SIZE+slen;
     plen += TAG_HDR_SIZE+slen;
 
+    /* If we asked for an MTU, handle it */
+    if (max_ppp_payload > ETH_PPPOE_MTU && ethif->mtu > 0) {
+	/* Shrink payload to fit */
+	if (max_ppp_payload > ethif->mtu - TOTAL_OVERHEAD) {
+	    max_ppp_payload = ethif->mtu - TOTAL_OVERHEAD;
+	}
+	if (max_ppp_payload > ETH_JUMBO_LEN - TOTAL_OVERHEAD) {
+	    max_ppp_payload = ETH_JUMBO_LEN - TOTAL_OVERHEAD;
+	}
+	if (max_ppp_payload > ETH_PPPOE_MTU) {
+	    PPPoETag maxPayload;
+	    UINT16_t mru = htons(max_ppp_payload);
+	    maxPayload.type = htons(TAG_PPP_MAX_PAYLOAD);
+	    maxPayload.length = htons(sizeof(mru));
+	    memcpy(maxPayload.payload, &mru, sizeof(mru));
+	    CHECK_ROOM(cursor, pads.payload, sizeof(mru) + TAG_HDR_SIZE);
+	    memcpy(cursor, &maxPayload, sizeof(mru) + TAG_HDR_SIZE);
+	    cursor += sizeof(mru) + TAG_HDR_SIZE;
+	    plen += sizeof(mru) + TAG_HDR_SIZE;
+	    cliSession->requested_mtu = max_ppp_payload;
+	}
+    }
+
     if (relayId.type) {
 	memcpy(cursor, &relayId, ntohs(relayId.length) + TAG_HDR_SIZE);
 	cursor += ntohs(relayId.length) + TAG_HDR_SIZE;
@@ -992,6 +1090,19 @@ processPADR(Interface *ethif, PPPoEPacket *packet, int len)
     pads.length = htons(plen);
     sendPacket(NULL, sock, &pads, (int) (plen + HDR_SIZE));
 
+    if (hurl_string || motd_string) {
+	memset(&conn, 0, sizeof(conn));
+	conn.hostUniq = NULL;
+
+	memcpy(conn.myEth, cliSession->ethif->mac, ETH_ALEN);
+	conn.discoverySocket = sock;
+	conn.session = cliSession->sess;
+	memcpy(conn.peerEth, cliSession->eth, ETH_ALEN);
+	if (hurl_string != NULL)
+	    sendHURLorMOTM(&conn, hurl_string, TAG_HURL);
+	if (motd_string != NULL)
+	    sendHURLorMOTM(&conn, motd_string, TAG_MOTM);
+    }
     /* Close sock; don't need it any more */
     close(sock);
 
@@ -1052,6 +1163,9 @@ usage(char const *argv0)
     fprintf(stderr, "   -o offset      -- Assign session numbers starting at offset+1.\n");
     fprintf(stderr, "   -f disc:sess   -- Set Ethernet frame types (hex).\n");
     fprintf(stderr, "   -s             -- Use synchronous PPP mode.\n");
+    fprintf(stderr, "   -X pidfile     -- Write PID and lock pidfile.\n");
+    fprintf(stderr, "   -q /path/pppd  -- Specify full path to pppd.\n");
+    fprintf(stderr, "   -Q /path/pppoe -- Specify full path to pppoe.\n");
 #ifdef HAVE_LINUX_KERNEL_PPPOE
     fprintf(stderr, "   -k             -- Use kernel-mode PPPoE.\n");
 #endif
@@ -1065,8 +1179,12 @@ usage(char const *argv0)
     fprintf(stderr, "   -1             -- Allow only one session per user.\n");
 #endif
 
+    fprintf(stderr, "   -i             -- Ignore PADI if no free sessions.\n");
+    fprintf(stderr, "   -M msg         -- Send MSG in a MOTM tag in PADM packet after PADS.\n");
+    fprintf(stderr, "   -H url         -- Send URL in a HURL tag in PADM packet after PADS.\n");
     fprintf(stderr, "   -h             -- Print usage information.\n\n");
-    fprintf(stderr, "PPPoE-Server Version %s, Copyright (C) 2001-2006 Roaring Penguin Software Inc.\n", VERSION);
+    fprintf(stderr, "PPPoE-Server Version %s, Copyright (C) 2001-2009 Roaring Penguin Software Inc.\n", RP_VERSION);
+    fprintf(stderr, "                     %*s  Copyright (C) 2018-2020 Dianne Skoll\n", (int) strlen(RP_VERSION), "");
 
 #ifndef HAVE_LICENSE
     fprintf(stderr, "PPPoE-Server comes with ABSOLUTELY NO WARRANTY.\n");
@@ -1074,7 +1192,7 @@ usage(char const *argv0)
     fprintf(stderr, "under the terms of the GNU General Public License, version 2\n");
     fprintf(stderr, "or (at your option) any later version.\n");
 #endif
-    fprintf(stderr, "http://www.roaringpenguin.com\n");
+    fprintf(stderr, "https://dianne.skoll.ca/projects/rp-pppoe/\n");
 }
 
 /**********************************************************************
@@ -1098,14 +1216,17 @@ main(int argc, char **argv)
     int found;
     unsigned int discoveryType, sessionType;
     char *addressPoolFname = NULL;
+    char *pidfile = NULL;
+    char c;
+
 #ifdef HAVE_LICENSE
     int use_clustering = 0;
 #endif
 
 #ifndef HAVE_LINUX_KERNEL_PPPOE
-    char *options = "x:hI:C:L:R:T:m:FN:f:O:o:sp:lrudPc:S:1";
+    char *options = "X:ix:hI:C:L:R:T:m:FN:f:O:o:sp:lrudPc:S:1q:Q:H:M:";
 #else
-    char *options = "x:hI:C:L:R:T:m:FN:f:O:o:skp:lrudPc:S:1";
+    char *options = "X:ix:hI:C:L:R:T:m:FN:f:O:o:skp:lrudPc:S:1q:Q:H:M:";
 #endif
 
     if (getuid() != geteuid() ||
@@ -1127,6 +1248,9 @@ main(int argc, char **argv)
     /* Parse command-line options */
     while((opt = getopt(argc, argv, options)) != -1) {
 	switch(opt) {
+	case 'i':
+	    IgnorePADIIfNoFreeSessions = 1;
+	    break;
 	case 'x':
 	    if (sscanf(optarg, "%d", &MaxSessionsPerMac) != 1) {
 		usage(argv[0]);
@@ -1155,6 +1279,44 @@ main(int argc, char **argv)
 	    }
 	    NumServiceNames++;
 	    break;
+	case 'q':
+	    pppd_path = strdup(optarg);
+	    if (!pppd_path) {
+		fprintf(stderr, "Out of memory");
+		exit(1);
+	    }
+	    break;
+	case 'Q':
+	    pppoe_path = strdup(optarg);
+	    if (!pppoe_path) {
+		fprintf(stderr, "Out of memory");
+		exit(1);
+	    }
+	    break;
+
+	case 'M':
+	    if (motd_string) {
+		free(motd_string);
+		motd_string = NULL;
+	    }
+	    motd_string = strdup(optarg);
+	    if (!motd_string) {
+		fprintf(stderr, "Out of memory");
+		exit(1);
+	    }
+	    break;
+
+	case 'H':
+	    if (hurl_string) {
+		free(hurl_string);
+		hurl_string = NULL;
+	    }
+	    hurl_string = strdup(optarg);
+	    if (!hurl_string) {
+		fprintf(stderr, "Out of memory");
+		exit(1);
+	    }
+	    break;
 	case 'c':
 #ifndef HAVE_LICENSE
 	    fprintf(stderr, "Clustering capability not available.\n");
@@ -1187,6 +1349,9 @@ main(int argc, char **argv)
 	    SET_STRING(addressPoolFname, optarg);
 	    break;
 
+	case 'X':
+	    SET_STRING(pidfile, optarg);
+	    break;
 	case 's':
 	    Synchronous = 1;
 	    /* Pass the Synchronous option on to pppoe */
@@ -1461,7 +1626,8 @@ main(int argc, char **argv)
 
     /* Open all the interfaces */
     for (i=0; i<NumInterfaces; i++) {
-	interfaces[i].sock = openInterface(interfaces[i].name, Eth_PPPOE_Discovery, interfaces[i].mac);
+	interfaces[i].mtu = 0;
+	interfaces[i].sock = openInterface(interfaces[i].name, Eth_PPPOE_Discovery, interfaces[i].mac, &interfaces[i].mtu);
     }
 
     /* Ignore SIGPIPE */
@@ -1473,12 +1639,6 @@ main(int argc, char **argv)
 	rp_fatal("Could not create EventSelector -- probably out of memory");
     }
 
-    /* Set signal handlers for SIGTERM and SIGINT */
-    if (Event_HandleSignal(event_selector, SIGTERM, termHandler) < 0 ||
-	Event_HandleSignal(event_selector, SIGINT, termHandler) < 0) {
-	fatalSys("Event_HandleSignal");
-    }
-
     /* Control channel */
 #ifdef HAVE_LICENSE
     if (control_init(argc, argv, event_selector)) {
@@ -1526,12 +1686,45 @@ main(int argc, char **argv)
 
     /* Daemonize -- UNIX Network Programming, Vol. 1, Stevens */
     if (beDaemon) {
+	if (pipe(KidPipe) < 0) {
+	    fatalSys("pipe");
+	}
 	i = fork();
 	if (i < 0) {
 	    fatalSys("fork");
 	} else if (i != 0) {
 	    /* parent */
-	    exit(EXIT_SUCCESS);
+	    close(KidPipe[1]);
+	    KidPipe[1] = -1;
+	    /* Wait for child to give the go-ahead */
+	    while(1) {
+		int r = read(KidPipe[0], &c, 1);
+		if (r == 0) {
+		    fprintf(stderr, "EOF from child - something went wrong; please check logs.\n");
+		    exit(EXIT_FAILURE);
+		}
+		if (r < 0) {
+		    if (errno == EINTR) continue;
+		    fatalSys("read");
+		}
+		break;
+	    }
+
+	    if (c == 'X') {
+		exit(EXIT_SUCCESS);
+	    }
+
+	    /* Read error message from child */
+	    while (1) {
+		int r = read(KidPipe[0], &c, 1);
+		if (r == 0) exit(EXIT_FAILURE);
+		if (r < 0) {
+		    if (errno == EINTR) continue;
+		    fatalSys("read");
+		}
+		fprintf(stderr, "%c", c);
+	    }
+	    exit(EXIT_FAILURE);
 	}
 	setsid();
 	signal(SIGHUP, SIG_IGN);
@@ -1544,6 +1737,11 @@ main(int argc, char **argv)
 
 	chdir("/");
 
+	if (KidPipe[0] >= 0) {
+	    close(KidPipe[0]);
+	    KidPipe[0] = -1;
+	}
+
 	/* Point stdin/stdout/stderr to /dev/null */
 	for (i=0; i<3; i++) {
 	    close(i);
@@ -1557,6 +1755,45 @@ main(int argc, char **argv)
 	}
     }
 
+    if (pidfile) {
+	FILE *foo = NULL;
+	if (KidPipe[1] >= 0) foo = fdopen(KidPipe[1], "w");
+	struct flock fl;
+	char buf[64];
+	fl.l_type = F_WRLCK;
+	fl.l_whence = SEEK_SET;
+	fl.l_start = 0;
+	fl.l_len = 0;
+	LockFD = open(pidfile, O_RDWR|O_CREAT, 0666);
+	if (LockFD < 0) {
+	    syslog(LOG_INFO, "Could not open PID file %s: %s", pidfile, strerror(errno));
+	    if (foo) fprintf(foo, "ECould not open PID file %s: %s\n", pidfile, strerror(errno));
+	    exit(1);
+	}
+	if (fcntl(LockFD, F_SETLK, &fl) < 0) {
+	    syslog(LOG_INFO, "Could not lock PID file %s: Is another process running?", pidfile);
+	    if (foo) fprintf(foo, "ECould not lock PID file %s: Is another process running?\n", pidfile);
+	    exit(1);
+	}
+	ftruncate(LockFD, 0);
+	snprintf(buf, sizeof(buf), "%lu\n", (unsigned long) getpid());
+	write(LockFD, buf, strlen(buf));
+	/* Do not close fd... use it to retain lock */
+    }
+
+    /* Set signal handlers for SIGTERM and SIGINT */
+    if (Event_HandleSignal(event_selector, SIGTERM, termHandler) < 0 ||
+	Event_HandleSignal(event_selector, SIGINT, termHandler) < 0) {
+	fatalSys("Event_HandleSignal");
+    }
+
+    /* Tell parent all is cool */
+    if (KidPipe[1] >= 0) {
+	write(KidPipe[1], "X", 1);
+	close(KidPipe[1]);
+	KidPipe[1] = -1;
+    }
+
     for(;;) {
 	i = Event_HandleEvent(event_selector);
 	if (i < 0) {
@@ -1586,6 +1823,17 @@ serverProcessPacket(Interface *i)
 	return;
     }
 
+    if (len < HDR_SIZE) {
+	/* Impossible - ignore */
+	return;
+    }
+
+    /* Sanity check on packet */
+    if (PPPOE_VER(packet.vertype) != 1 || PPPOE_TYPE(packet.vertype) != 1) {
+	/* Syslog an error */
+	return;
+    }
+
     /* Check length */
     if (ntohs(packet.length) + HDR_SIZE > len) {
 	syslog(LOG_ERR, "Bogus PPPoE length field (%u)",
@@ -1593,11 +1841,6 @@ serverProcessPacket(Interface *i)
 	return;
     }
 
-    /* Sanity check on packet */
-    if (packet.ver != 1 || packet.type != 1) {
-	/* Syslog an error */
-	return;
-    }
     switch(packet.code) {
     case CODE_PADI:
 	processPADI(i, &packet, len);
@@ -1651,8 +1894,7 @@ sendErrorPADS(int sock,
     memcpy(pads.ethHdr.h_dest, dest, ETH_ALEN);
     memcpy(pads.ethHdr.h_source, source, ETH_ALEN);
     pads.ethHdr.h_proto = htons(Eth_PPPOE_Discovery);
-    pads.ver = 1;
-    pads.type = 1;
+    pads.vertype = PPPOE_VER_TYPE(1, 1);
     pads.code = CODE_PADS;
 
     pads.session = htons(0);
@@ -1694,9 +1936,9 @@ void
 startPPPDUserMode(ClientSession *session)
 {
     /* Leave some room */
-    char *argv[32];
+    char *argv[64];
 
-    char buffer[SMALLBUF];
+    char buffer[2 * SMALLBUF];
 
     int c = 0;
 
@@ -1704,8 +1946,8 @@ startPPPDUserMode(ClientSession *session)
     argv[c++] = "pty";
 
     /* Let's hope service-name does not have ' in it... */
-    snprintf(buffer, SMALLBUF, "%s -n -I %s -e %u:%02x:%02x:%02x:%02x:%02x:%02x%s -S '%s'",
-	     PPPOE_PATH, session->ethif->name,
+    snprintf(buffer, sizeof(buffer), "%s -n -I %s -e %u:%02x:%02x:%02x:%02x:%02x:%02x%s -S '%s'",
+	     pppoe_path, session->ethif->name,
 	     (unsigned int) ntohs(session->sess),
 	     session->eth[0], session->eth[1], session->eth[2],
 	     session->eth[3], session->eth[4], session->eth[5],
@@ -1719,7 +1961,7 @@ startPPPDUserMode(ClientSession *session)
     argv[c++] = "file";
     argv[c++] = pppoptfile;
 
-    snprintf(buffer, SMALLBUF, "%d.%d.%d.%d:%d.%d.%d.%d",
+    snprintf(buffer, sizeof(buffer), "%d.%d.%d.%d:%d.%d.%d.%d",
 	    (int) session->myip[0], (int) session->myip[1],
 	    (int) session->myip[2], (int) session->myip[3],
 	    (int) session->peerip[0], (int) session->peerip[1],
@@ -1740,23 +1982,32 @@ startPPPDUserMode(ClientSession *session)
     }
     argv[c++] = "nodetach";
     argv[c++] = "noaccomp";
-    argv[c++] = "nobsdcomp";
-    argv[c++] = "nodeflate";
     argv[c++] = "nopcomp";
-    argv[c++] = "novj";
-    argv[c++] = "novjccomp";
     argv[c++] = "default-asyncmap";
     if (Synchronous) {
 	argv[c++] = "sync";
     }
     if (PassUnitOptionToPPPD) {
 	argv[c++] = "unit";
-	sprintf(buffer, "%u", (unsigned int) (ntohs(session->sess) - 1 - SessOffset));
+	sprintf(buffer, "%u", (unsigned int) (ntohs(session->sess) - 1));
 	argv[c++] = buffer;
     }
+    if (session->requested_mtu > 1492) {
+	sprintf(buffer, "%u", (unsigned int) session->requested_mtu);
+	argv[c++] = "mru";
+	argv[c++] = buffer;
+	argv[c++] = "mtu";
+	argv[c++] = buffer;
+    } else {
+	argv[c++] = "mru";
+	argv[c++] = "1492";
+	argv[c++] = "mtu";
+	argv[c++] = "1492";
+    }
+
     argv[c++] = NULL;
 
-    execv(PPPD_PATH, argv);
+    execv(pppd_path, argv);
     exit(EXIT_FAILURE);
 }
 
@@ -1826,19 +2077,27 @@ startPPPDLinuxKernelMode(ClientSession *session)
     }
     argv[c++] = "nodetach";
     argv[c++] = "noaccomp";
-    argv[c++] = "nobsdcomp";
-    argv[c++] = "nodeflate";
     argv[c++] = "nopcomp";
-    argv[c++] = "novj";
-    argv[c++] = "novjccomp";
     argv[c++] = "default-asyncmap";
     if (PassUnitOptionToPPPD) {
 	argv[c++] = "unit";
 	sprintf(buffer, "%u", (unsigned int) (ntohs(session->sess) - 1 - SessOffset));
 	argv[c++] = buffer;
     }
+    if (session->requested_mtu > 1492) {
+	sprintf(buffer, "%u", (unsigned int) session->requested_mtu);
+	argv[c++] = "mru";
+	argv[c++] = buffer;
+	argv[c++] = "mtu";
+	argv[c++] = buffer;
+    } else {
+	argv[c++] = "mru";
+	argv[c++] = "1492";
+	argv[c++] = "mtu";
+	argv[c++] = "1492";
+    }
     argv[c++] = NULL;
-    execv(PPPD_PATH, argv);
+    execv(pppd_path, argv);
     exit(EXIT_FAILURE);
 }
 
@@ -1897,7 +2156,7 @@ PppoeStopSession(ClientSession *ses,
     PPPoEConnection conn;
 
     memset(&conn, 0, sizeof(conn));
-    conn.useHostUniq = 0;
+    conn.hostUniq = NULL;
 
     memcpy(conn.myEth, ses->ethif->mac, ETH_ALEN);
     conn.discoverySocket = ses->ethif->sock;
@@ -2002,6 +2261,7 @@ pppoe_alloc_session(void)
     ses->flags = 0;
     ses->startTime = time(NULL);
     ses->serviceName = "";
+    ses->requested_mtu = 0;
 #ifdef HAVE_LICENSE
     memset(ses->user, 0, MAX_USERNAME_LEN+1);
     memset(ses->realm, 0, MAX_USERNAME_LEN+1);
@@ -2061,6 +2321,7 @@ pppoe_free_session(ClientSession *ses)
     /* Initialize fields to sane values */
     ses->funcs = &DefaultSessionFunctionTable;
     ses->pid = 0;
+    memset(ses->eth, 0, ETH_ALEN);
     ses->flags = 0;
 #ifdef HAVE_L2TP
     ses->l2tp_ses = NULL;
@@ -2094,8 +2355,8 @@ sendHURLorMOTM(PPPoEConnection *conn, char const *url, UINT16_t tag)
     if (conn->discoverySocket < 0) return;
 
     if (tag == TAG_HURL) {
-	if (strncmp(url, "http://", 7)) {
-	    syslog(LOG_WARNING, "sendHURL(%s): URL must begin with http://", url);
+	if (strncmp(url, "http://", 7) && strncmp(url, "https://", 8)) {
+	    syslog(LOG_WARNING, "sendHURL(%s): URL must begin with http:// or https://", url);
 	    return;
 	}
     } else {
@@ -2106,8 +2367,7 @@ sendHURLorMOTM(PPPoEConnection *conn, char const *url, UINT16_t tag)
     memcpy(packet.ethHdr.h_source, conn->myEth, ETH_ALEN);
 
     packet.ethHdr.h_proto = htons(Eth_PPPOE_Discovery);
-    packet.ver = 1;
-    packet.type = 1;
+    packet.vertype = PPPOE_VER_TYPE(1, 1);
     packet.code = CODE_PADM;
     packet.session = conn->session;
 
