@@ -5,7 +5,8 @@
 * Sniff a network for likely-looking PPPoE frames and deduce the value
 * to supply to PPPOE_EXTRA in /etc/ppp/pppoe.conf.  USE AT YOUR OWN RISK.
 *
-* Copyright (C) 2000 by Roaring Penguin Software Inc.
+* Copyright (C) 2000-2018 by Roaring Penguin Software Inc.
+* Copyright (C) 2018-2020 Dianne Skoll
 *
 * This program may be distributed according to the terms of the GNU
 * General Public License, version 2 or (at your option) any later version.
@@ -14,9 +15,7 @@
 *
 ***********************************************************************/
 
-static char const RCSID[] =
-"$Id$";
-
+#define _GNU_SOURCE 1
 #include "pppoe.h"
 
 #ifdef HAVE_GETOPT_H
@@ -124,11 +123,12 @@ usage(char const *argv0)
     fprintf(stderr, "   -I if_name     -- Specify interface (default %s.)\n",
 	    DEFAULT_IF);
     fprintf(stderr, "   -V             -- Print version and exit.\n");
-    fprintf(stderr, "\nPPPoE Version %s, Copyright (C) 2000 Roaring Penguin Software Inc.\n", VERSION);
+    fprintf(stderr, "\nPPPoE Version %s, Copyright (C) 2000 Roaring Penguin Software Inc.\n", RP_VERSION);
+    fprintf(stderr, "              %*s  Copyright (C) 2018-2020 Dianne Skoll\n", (int) strlen(RP_VERSION), "");
     fprintf(stderr, "PPPoE comes with ABSOLUTELY NO WARRANTY.\n");
     fprintf(stderr, "This is free software, and you are welcome to redistribute it under the terms\n");
     fprintf(stderr, "of the GNU General Public License, version 2 or any later version.\n");
-    fprintf(stderr, "http://www.roaringpenguin.com\n");
+    fprintf(stderr, "https://dianne.skoll.ca/projects/rp-pppoe/\n");
     exit(0);
 }
 
@@ -175,7 +175,7 @@ main(int argc, char *argv[])
 	    SET_STRING(IfName, optarg);
 	    break;
 	case 'V':
-	    printf("pppoe-sniff: Roaring Penguin PPPoE Version %s\n", VERSION);
+	    printf("pppoe-sniff: RP-PPPoE Version %s\n", RP_VERSION);
 	    exit(0);
 	default:
 	    usage(argv[0]);
@@ -206,7 +206,7 @@ main(int argc, char *argv[])
     while (!SeenPADR) {
 	if (receivePacket(sock, &pkt, &size) < 0) continue;
 	if (ntohs(pkt.length) + HDR_SIZE > size) continue;
-	if (pkt.ver != 1 || pkt.type != 1)       continue;
+	if (PPPOE_VER(pkt.vertype) != 1 || PPPOE_TYPE(pkt.vertype) != 1) continue;
 	if (pkt.code != CODE_PADR)               continue;
 
 	/* Looks promising... parse it */
@@ -221,7 +221,7 @@ main(int argc, char *argv[])
     while (!SeenSess) {
 	if (receivePacket(sock, &pkt, &size) < 0) continue;
 	if (ntohs(pkt.length) + HDR_SIZE > size) continue;
-	if (pkt.ver != 1 || pkt.type != 1)       continue;
+	if (PPPOE_VER(pkt.vertype) != 1 || PPPOE_TYPE(pkt.vertype) != 1) continue;
 	if (pkt.code != CODE_SESS)               continue;
 
 	/* Cool! */
